@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require 'puma/runner'
 require 'puma/util'
 require 'puma/plugin'
@@ -17,8 +19,6 @@ module Puma
   # via the `spawn_workers` method call. Each worker will have it's own
   # instance of a `Puma::Server`.
   class Cluster < Runner
-    WORKER_CHECK_INTERVAL = 5
-
     def initialize(cli, events)
       super cli, events
 
@@ -35,7 +35,11 @@ module Puma
       @workers.each { |x| x.term }
 
       begin
-        @workers.each { |w| Process.waitpid(w.pid) }
+        loop do
+          wait_workers
+          break if @workers.empty?
+          sleep 0.2
+        end
       rescue Interrupt
         log "! Cancelled waiting for workers"
       end
@@ -67,12 +71,13 @@ module Puma
         @signal = "TERM"
         @options = options
         @first_term_sent = nil
+        @started_at = Time.now
         @last_checkin = Time.now
         @last_status = '{}'
-        @dead = false
+        @term = false
       end
 
-      attr_reader :index, :pid, :phase, :signal, :last_checkin, :last_status
+      attr_reader :index, :pid, :phase, :signal, :last_checkin, :last_status, :started_at
 
       def booted?
         @stage == :booted
@@ -83,12 +88,8 @@ module Puma
         @stage = :booted
       end
 
-      def dead?
-        @dead
-      end
-
-      def dead!
-        @dead = true
+      def term?
+        @term
       end
 
       def ping!(status)
@@ -105,9 +106,9 @@ module Puma
           if @first_term_sent && (Time.now - @first_term_sent) > @options[:worker_shutdown_timeout]
             @signal = "KILL"
           else
+            @term ||= true
             @first_term_sent ||= Time.now
           end
-
           Process.kill @signal, @pid
         rescue Errno::ESRCH
         end
@@ -181,7 +182,7 @@ module Puma
     def check_workers(force=false)
       return if !force && @next_check && @next_check >= Time.now
 
-      @next_check = Time.now + WORKER_CHECK_INTERVAL
+      @next_check = Time.now + Const::WORKER_CHECK_INTERVAL
 
       any = false
 
@@ -198,15 +199,7 @@ module Puma
       # during this loop by giving the kernel time to kill them.
       sleep 1 if any
 
-      while @workers.any?
-        pid = Process.waitpid(-1, Process::WNOHANG)
-        break unless pid
-
-        @workers.delete_if { |w| w.pid == pid }
-      end
-
-      @workers.delete_if(&:dead?)
-
+      wait_workers
       cull_workers
       spawn_workers
 
@@ -223,8 +216,10 @@ module Puma
             log "- Stopping #{w.pid} for phased upgrade..."
           end
 
-          w.term
-          log "- #{w.signal} sent to #{w.pid}..."
+          unless w.term?
+            w.term
+            log "- #{w.signal} sent to #{w.pid}..."
+          end
         end
       end
     end
@@ -251,6 +246,7 @@ module Puma
       @suicide_pipe.close
 
       Thread.new do
+        Puma.set_thread_name "worker check pipe"
         IO.select [@check_pipe]
         log "! Detected parent died, dying"
         exit! 1
@@ -273,6 +269,7 @@ module Puma
       server = start_server
 
       Signal.trap "SIGTERM" do
+        @worker_write << "e#{Process.pid}\n" rescue nil
         server.stop
       end
 
@@ -285,10 +282,11 @@ module Puma
       end
 
       Thread.new(@worker_write) do |io|
+        Puma.set_thread_name "stat payload"
         base_payload = "p#{Process.pid}"
 
         while true
-          sleep WORKER_CHECK_INTERVAL
+          sleep Const::WORKER_CHECK_INTERVAL
           begin
             b = server.backlog || 0
             r = server.running || 0
@@ -350,11 +348,13 @@ module Puma
       Dir.chdir dir
     end
 
+    # Inside of a child process, this will return all zeroes, as @workers is only populated in
+    # the master process.
     def stats
       old_worker_count = @workers.count { |w| w.phase != @phase }
       booted_worker_count = @workers.count { |w| w.booted? }
-      worker_status = '[' + @workers.map { |w| %Q!{ "pid": #{w.pid}, "index": #{w.index}, "phase": #{w.phase}, "booted": #{w.booted?}, "last_checkin": "#{w.last_checkin.utc.iso8601}", "last_status": #{w.last_status} }!}.join(",") + ']'
-      %Q!{ "workers": #{@workers.size}, "phase": #{@phase}, "booted_workers": #{booted_worker_count}, "old_workers": #{old_worker_count}, "worker_status": #{worker_status} }!
+      worker_status = '[' + @workers.map { |w| %Q!{ "started_at": "#{w.started_at.utc.iso8601}", "pid": #{w.pid}, "index": #{w.index}, "phase": #{w.phase}, "booted": #{w.booted?}, "last_checkin": "#{w.last_checkin.utc.iso8601}", "last_status": #{w.last_status} }!}.join(",") + ']'
+      %Q!{ "started_at": "#{@started_at.utc.iso8601}", "workers": #{@workers.size}, "phase": #{@phase}, "booted_workers": #{booted_worker_count}, "old_workers": #{old_worker_count}, "worker_status": #{worker_status} }!
     end
 
     def preload?
@@ -388,10 +388,13 @@ module Puma
           log "Early termination of worker"
           exit! 0
         else
+          @launcher.close_binder_listeners
+
           stop_workers
           stop
 
-          raise SignalException, "SIGTERM"
+          raise(SignalException, "SIGTERM") if @options[:raise_exception_on_sigterm]
+          exit 0 # Clean exit, workers were stopped
         end
       end
     end
@@ -485,7 +488,7 @@ module Puma
 
             force_check = false
 
-            res = IO.select([read], nil, nil, WORKER_CHECK_INTERVAL)
+            res = IO.select([read], nil, nil, Const::WORKER_CHECK_INTERVAL)
 
             if res
               req = read.read_nonblock(1)
@@ -501,8 +504,11 @@ module Puma
                   w.boot!
                   log "- Worker #{w.index} (pid: #{pid}) booted, phase: #{w.phase}"
                   force_check = true
+                when "e"
+                  # external term, see worker method, Signal.trap "SIGTERM"
+                  w.instance_variable_set :@term, true
                 when "t"
-                  w.dead!
+                  w.term unless w.term?
                   force_check = true
                 when "p"
                   w.ping!(result.sub(/^\d+/,'').chomp)
@@ -525,5 +531,24 @@ module Puma
         @wakeup.close
       end
     end
+
+    private
+
+    # loops thru @workers, removing workers that exited, and calling
+    # `#term` if needed
+    def wait_workers
+      @workers.reject! do |w|
+        begin
+          if Process.wait(w.pid, Process::WNOHANG)
+            true
+          else
+            w.term if w.term?
+            nil
+          end
+        rescue Errno::ECHILD
+          true # child is already terminated
+        end
+      end
+    end
   end
 end
