@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 # Copyright (c) 2011 Evan Phoenix
 # Copyright (c) 2005 Zed A. Shaw
 
@@ -5,25 +6,24 @@ if %w(2.2.7 2.2.8 2.2.9 2.2.10 2.3.4 2.4.1).include? RUBY_VERSION
   begin
     require 'stopgap_13632'
   rescue LoadError
+    puts "For test stability, you must install the stopgap_13632 gem."
+    exit(1)
   end
 end
 
-begin
-  require "bundler/setup"
-  # bundler/setup may not load bundler
-  require "bundler" unless Bundler.const_defined?(:ORIGINAL_ENV)
-rescue LoadError
-  warn "Failed to load bundler ... this should only happen during package building"
-end
-
 require "net/http"
 require "timeout"
 require "minitest/autorun"
 require "minitest/pride"
+require "minitest/proveit"
+require_relative "helpers/apps"
 
 $LOAD_PATH << File.expand_path("../../lib", __FILE__)
 Thread.abort_on_exception = true
 
+$debugging_info = ''.dup
+$debugging_hold = false    # needed for TestCLI#test_control_clustered
+
 require "puma"
 require "puma/events"
 require "puma/detect"
@@ -47,10 +47,14 @@ end
 
 module UniquePort
   @port  = 3211
+  @mutex = Mutex.new
 
   def self.call
-    @port += 1
-    @port
+    @mutex.synchronize {
+      @port += 1
+      @port = 3307 if @port == 3306  # MySQL on Actions
+      @port
+    }
   end
 end
 
@@ -71,7 +75,28 @@ if ENV['CI']
   Minitest::Retry.use!
 end
 
-module SkipTestsBasedOnRubyEngine
+module TestSkips
+
+  # usage: skip NO_FORK_MSG unless HAS_FORK
+  # windows >= 2.6 fork is not defined, < 2.6 fork raises NotImplementedError
+  HAS_FORK = ::Process.respond_to? :fork
+  NO_FORK_MSG = "Kernel.fork isn't available on the #{RUBY_PLATFORM} platform"
+
+  # socket is required by puma
+  # usage: skip UNIX_SKT_MSG unless UNIX_SKT_EXIST
+  UNIX_SKT_EXIST = Object.const_defined? :UNIXSocket
+  UNIX_SKT_MSG = "UnixSockets aren't available on the #{RUBY_PLATFORM} platform"
+
+  SIGNAL_LIST = Signal.list.keys.map(&:to_sym) - (Puma.windows? ? [:INT, :TERM] : [])
+
+  # usage: skip_unless_signal_exist? :USR2
+  def skip_unless_signal_exist?(sig, bt: caller)
+    signal = sig.to_s.sub(/\ASIG/, '').to_sym
+    unless SIGNAL_LIST.include? signal
+      skip "Signal #{signal} isn't available on the #{RUBY_PLATFORM} platform", bt
+    end
+  end
+
   # called with one or more params, like skip_on :jruby, :windows
   # optional suffix kwarg is appended to the skip message
   # optional suffix bt should generally not used
@@ -79,10 +104,11 @@ module SkipTestsBasedOnRubyEngine
     skip_msg = false
     engs.each do |eng|
       skip_msg = case eng
+        when :darwin   then "Skipped on darwin#{suffix}"    if RUBY_PLATFORM[/darwin/]
         when :jruby    then "Skipped on JRuby#{suffix}"     if Puma.jruby?
         when :windows  then "Skipped on Windows#{suffix}"   if Puma.windows?
-        when :appveyor then "Skipped on Appveyor#{suffix}"  if ENV["APPVEYOR"]
         when :ci       then "Skipped on ENV['CI']#{suffix}" if ENV["CI"]
+        when :no_bundler then "Skipped w/o Bundler#{suffix}"  if !defined?(Bundler)
         else false
       end
       skip skip_msg, bt if skip_msg
@@ -92,6 +118,7 @@ module SkipTestsBasedOnRubyEngine
   # called with only one param
   def skip_unless(eng, bt: caller)
     skip_msg = case eng
+      when :darwin  then "Skip unless darwin"  unless RUBY_PLATFORM[/darwin/]
       when :jruby   then "Skip unless JRuby"   unless Puma.jruby?
       when :windows then "Skip unless Windows" unless Puma.windows?
       else false
@@ -100,4 +127,26 @@ module SkipTestsBasedOnRubyEngine
   end
 end
 
-Minitest::Test.include SkipTestsBasedOnRubyEngine
+Minitest::Test.include TestSkips
+
+class Minitest::Test
+  def self.run(reporter, options = {}) # :nodoc:
+    prove_it!
+    super
+  end
+
+  def full_name
+    "#{self.class.name}##{name}"
+  end
+end
+
+Minitest.after_run do
+  # needed for TestCLI#test_control_clustered
+  unless $debugging_hold
+    out = $debugging_info.strip
+    unless out.empty?
+      puts "", " Debugging Info".rjust(75, '-'),
+        out, '-' * 75, ""
+    end
+  end
+end
