@@ -1,4 +1,4 @@
-system "ruby -rubygems -I../../lib ../../bin/puma -p 10102 -C t3_conf.rb ../hello.ru &"
+system "ruby -rrubygems -Ilib bin/puma -p 10102 -C test/shell/t3_conf.rb test/rackup/hello.ru &"
 sleep 5
 
 worker_pid_was_present = File.file? "t3-worker-2-pid"
@@ -9,7 +9,7 @@ sleep 2
 
 worker_index_within_number_of_workers = !File.file?("t3-worker-3-pid")
 
-system "kill `cat t3-pid`" 
+system "kill `cat t3-pid`"
 
 File.unlink "t3-pid" if File.file? "t3-pid"
 File.unlink "t3-worker-0-pid" if File.file? "t3-worker-0-pid"
@@ -22,4 +22,3 @@ if worker_pid_was_present and worker_index_within_number_of_workers
 else
   exit 1
 end
-
