@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require_relative "helper"
 
 class TestPumaUnixSocket < Minitest::Test
@@ -7,19 +9,19 @@ class TestPumaUnixSocket < Minitest::Test
   Path = "test/puma.sock"
 
   def setup
-    # UNIX sockets are not recommended on JRuby or Windows
-    skip_on :jruby, :windows, suffix: " - UNIX sockets are not recommended"
+    return unless UNIX_SKT_EXIST
     @server = Puma::Server.new App
     @server.add_unix_listener Path
     @server.run
   end
 
   def teardown
-    @server.stop(true) if @server
-    File.unlink Path if File.exist? Path
+    return unless UNIX_SKT_EXIST
+    @server.stop(true)
   end
 
   def test_server
+    skip UNIX_SKT_MSG unless UNIX_SKT_EXIST
     sock = UNIXSocket.new Path
 
     sock << "GET / HTTP/1.0\r\nHost: blah.com\r\n\r\n"
@@ -27,7 +29,5 @@ class TestPumaUnixSocket < Minitest::Test
     expected = "HTTP/1.0 200 OK\r\nContent-Length: 5\r\n\r\nWorks"
 
     assert_equal expected, sock.read(expected.size)
-
-    sock.close
   end
 end
